/*******************
VARIABLES
*******************/
var creativeId = "Banner"; //Use anything that uniquely identifies this creative
var creativeVersion = "2.1.0"; //0.0.1 during initial dev, 1.0.0 on release, x.x.x once released and receives updates
var lastModified = "2017-07-19";
var lastUploaded = "2017-07-19";

var templateVersion = "2.0.23";

var adId, rnd, uid;
var bannerDiv;
var expandButton;
var expandType;
var panelName;
var initPanelHeight;
var panelWidth;
var panelHeight;
var PANEL_HEIGHT_DEPENDENCY;
var panelAutoBreakPoints;
var panelUserBreakPoints;
var panelActionType;
var parentViewPort;
var initComplete;
var ccReady;
var sdkData;
var isMRAID;

/*******************
INITIALIZATION
*******************/
function checkIfAdKitReady(event) {
	try {
		if (window.localPreview) {
			window.initializeLocalPreview(); //in localPreview.js
			initializeCreative();
			return;
		}
	}
	catch (e) {}

	adkit.onReady(initializeCreative);
}

function initializeCreative(event) {
	// so messaging can work in safe frames we need to bind the events that are present in the event manager.
	EBG.pm.bind("sendCreativeId", function() {
		eventManager.apply(this, arguments);
	}, this);
	EBG.pm.bind("eventCallback", function() {
		eventManager.apply(this, arguments);
	}, this);

	EB.Comm = EB.Comm || new EBG.Comm();
	EB.Comm.setName("banner");

	//Workaround (from QB6573) for Async EB Load where Modernizr isn't properly initialized
	typeof Modernizr === "object" && (Modernizr.touch = Modernizr.touch || "ontouchstart" in window);

	window.registerInteraction = function() {}; //overwrite rI function because it will never actually be called
	initializeGlobalVariables();
	initializeCloseButton();
	addEventListeners();
	setCreativeVersion();

	dispatchCustomScriptEvent("compatibilityMode", {
		mode: false
	}); //do not remove this line - allows for backwards compatibility

	if (isMRAID) {
		document.addEventListener("touchmove", function(event){
			event.preventDefault();
		});
	}

	initComplete = true;
}

function initializeGlobalVariables() {
	adId = EB._adConfig.adId;
	rnd = EB._adConfig.rnd;
	uid = EB._adConfig.uid;
	creativeIFrameId = EB.assetId;

	bannerDiv = document.getElementById("banner");
	expandButton = document.getElementById("expandButton");

	if (!isMRAID) {
		sdkData = EB.getSDKData();
		isMRAID = sdkData !==null && sdkData.SDKType === "MRAID";
	}
}

function initializeCloseButton() {
	useCustomClose = !isMRAID || (isMRAID && !getCustomVar("mdEnableSDKDefaultCloseButton"));
	EB.setExpandProperties({
		useCustomClose: useCustomClose
	});
}

function addEventListeners() {
	expandButton.addEventListener("click", handleExpandButtonClick);
}

function registerInteraction() {
	//Register your automatic and user interactions here and the platform will parse them on workspace upload. This function will not be called.
	//Example: 'Left_Gutter_Viewed'

	//EB.automaticEventCounter("interactionName1");
	//EB.automaticEventCounter("interactionName2");

	//EB.userActionCounter("userActionName1");
	//EB.userActionCounter("userActionName2");
}

/*******************
EVENT HANDLERS
*******************/
function handleExpandButtonClick() {
	openPanel(EBG.ActionType.USER);
}

/*******************
UTILITIES
*******************/
function showMessage(msg) {
	// listen to comm module call back
}

function panelListener(event) {
	if (event.action === "AnimationStart") {
		EB.API.setStyle(expandButton, {
			display: "none"
		});
	}
	if (event.action === "AnimationComplete") {
		EB.API.setStyle(expandButton, {
			display: "block"
		});
	}
}

function getCustomVar(cv) {
	cv = EB.API.getCustomVar(cv);
	if (typeof(cv) != "undefined") {
		return cv;
	}
	else {
		return "noVarFound";
	}
}

function setCreativeVersion() {
	sendMessage("setCreativeVersion", {
		creativeId: creativeId,
		creativeVersion: creativeVersion,
		creativeLastModified: lastModified,
		uid: uid
	});
}

function creativeContainerReady() {
	if (ccReady) {
		return;
	}

	if (!initComplete) {
		setTimeout(creativeContainerReady, 500);
		return;
	}
	else {
		if (getCustomVar("mdIsAutoExpand")) {
			openPanel(EBG.ActionType.AUTO);
		}
		ccReady = true;
	}
}

function openPanel(actionType) {
	var panelName = actionType === EBG.ActionType.AUTO ? getCustomVar("mdAutoPanelName") : getCustomVar("mdUserPanelName");
	var expandType = actionType === EBG.ActionType.AUTO ? getCustomVar("mdAutoPanelType") : getCustomVar("mdUserPanelType");

	if (isMRAID) {
		EB.expand({
			panelName: panelName,
			width: 1,
			height: 1,
			actionType: actionType
		});
	}
	else {
		panelActionType = actionType;
		var push = false;
		var init = {};

		if (expandType.indexOf("pushdown") != -1) {
			push = {
				deltaOffset: 0
			};
		}

		if (expandType.indexOf("fullscreen") != -1) {
			init.top = 0;
			init.left = 0;
			panelWidth = parentViewPort.width;
			initPanelHeight = 0;
			if (!EB.API.os.mobile) {
				panelWidth += 17;
			}
			panelHeight = parentViewPort.height;
		}
		else {
			panelHeight = getPanelHeight(actionType);
		}

		init.clip = "rect(" + initPanelHeight + "px," + Math.max(EB._adConfig.width, panelWidth) + "px, 0px, 0px)";
		init.width = panelWidth;

		EB.expand({
			animate: {
				init: init,
				clip: "rect(0px," + panelWidth + "px," + panelHeight + "px, 0px)",
				width: panelWidth + "px",
				height: panelHeight + "px",
				duration: getCustomVar("mdAnimationDuration") * 1000,
				easing: getCustomVar("mdEasingType"),
				customData: {
					action: expandType
				}
			},
			panelName: panelName,
			pushdown: push,
			actionType: panelActionType
		});
	}
}

function getPanelHeight(actionType) {
	var panelHeightDependency = getCustomVar("mdPanelHeightDependency");
	var newPanelHeight = actionType === EBG.ActionType.AUTO ? getCustomVar("mdAutoPanelFixedHeight") : getCustomVar("mdUserPanelFixedHeight");;
	switch (panelHeightDependency.toLowerCase()) {
		case PANEL_HEIGHT_DEPENDENCY.FIXED:
			// keep use newPanelHeight defiend above
			break;
		case PANEL_HEIGHT_DEPENDENCY.ADAPTIVE:
			var breakPoints = panelActionType === EBG.ActionType.AUTO ? panelAutoBreakPoints : panelUserBreakPoints;
			if (breakPoints !== undefined) {
				var breakPointWidth = [];
				var itemCount = 0;
				for (var items in breakPoints) {
					if (breakPoints[items]) {
						breakPointWidth[itemCount] = items;
						itemCount++;
					}
				}

				breakPointWidth.sort(function compareNumbers(a, b) {
					return a - b;
				});

				var panelHeight = breakPoints[breakPointWidth[0]].height;
				for (var i = 0; i < breakPointWidth.length; i++) {
					if (panelWidth >= breakPointWidth[i]) {
						panelHeight = breakPoints[breakPointWidth[i]].height;
					}
				}
				newPanelHeight = panelHeight;
			}
			break;
		case PANEL_HEIGHT_DEPENDENCY.ASPECT_RATIO:
			var panelAspectRatio = getCustomVar("mdPanelAspectRatio");
			if (panelAspectRatio !== undefined && panelAspectRatio.match(/\d+:\d+/) !== null) {
				newPanelHeight = parseInt((panelWidth / panelAspectRatio.split(":")[0]) * panelAspectRatio.split(":")[1])
			}
			break;
		case PANEL_HEIGHT_DEPENDENCY.BROWSER:
			newPanelHeight = parentViewPort.height;
			break;
		case PANEL_HEIGHT_DEPENDENCY.MATCH_BANNER:
			newPanelHeight = initPanelHeight;
			break;
	}
	return newPanelHeight;
}

/*********************************
HTML5 Event System - Do Not Modify
*********************************/
var listenerQueue;
var creativeIFrameId;

function sendMessage(type, data) {
	if (!data.type) {
		data.type = type;
	}
	EB._sendMessage(type, data);
}

function addCustomScriptEventListener(eventName, callback, interAd) {
	listenerQueue = listenerQueue || {};
	var data = {
		uid: uid,
		listenerId: Math.ceil(Math.random() * 1000000000),
		eventName: eventName,
		interAd: !!(interAd),
		creativeIFrameId: creativeIFrameId
	};
	sendMessage("addCustomScriptEventListener", data);
	data.callback = callback;
	listenerQueue[data.listenerId] = data;
	return data.listenerId;
}

function dispatchCustomScriptEvent(eventName, params) {
	params = params || {};
	params.uid = uid;
	params.eventName = eventName;
	params.creativeIFrameId = creativeIFrameId;
	sendMessage("dispatchCustomScriptEvent", params);
}

function removeCustomScriptEventListener(listenerId) {
	var params = {
		uid: uid,
		listenerId: listenerId,
		creativeIFrameId: creativeIFrameId
	};

	sendMessage("removeCustomScriptEventListener", params);
	if (listenerQueue[listenerId]) {
		delete listenerQueue[listenerId];
	}
}

function eventManager(event) {
	var msg;
	if (typeof event == "object" && event.data) {
		msg = JSON.parse(event.data);
	}
	else {
		// this is safe frame.
		msg = {
			type: event.type,
			data: event
		};
	}
	if (msg.type && msg.data && (!uid || (msg.data.uid && msg.data.uid == uid))) {
		switch (msg.type) {
			case "sendCreativeId":
				creativeIFrameId = msg.data.creativeIFrameId;
				isMRAID = msg.data.mraid;
				parentViewPort = msg.data.viewport;
				initPanelHeight = msg.data.bannerHeight;
				panelWidth = msg.data.bannerWidth;
				panelAutoBreakPoints = msg.data.panelAutoBreakPoints;
				panelUserBreakPoints = msg.data.panelUserBreakPoints;
				PANEL_HEIGHT_DEPENDENCY = msg.data.panelHeightDependency;
				addCustomScriptEventListener("panelBehaviour", panelListener, false);
				addCustomScriptEventListener("parentViewPort", function(event) {
					parentViewPort = event.viewport;
				}, false);
				addCustomScriptEventListener("currentExpandData", function(event) {
					panelWidth = event.bannerWidth;
					initPanelHeight = event.bannerHeight;
				}, false);
				if (creativeContainerReady)
					creativeContainerReady();
				break;
			case "animationComplete":
				panelListener({
					action: msg.data.action,
					expanding: msg.data.expanding
				});
				break;
			case "eventCallback": // Handle Callback
				var list = msg.data.listenerIds;
				var length = list.length;
				for (var i = 0; i < length; i++) {
					try {
						var t = listenerQueue[list[i]];
						if (!t) continue;
						t.callback(msg.data);
					}
					catch (e) {}
				}
				break;
		}
	}
}

window.addEventListener("message", function() {
	try {
		eventManager.apply(this, arguments);
	}
	catch (e) {}
}, false);
/*************************************
End HTML5 Event System - Do Not Modify
*************************************/

window.addEventListener("load", checkIfAdKitReady);
