//******************************************************************************
// This class is part of the Sizmek Outstream Player Ad Feature
// ALL RIGHTS RESERVED TO © 2017 Sizmek.
//******************************************************************************
//******************************************************************************

(function() {
    // Set Scope
    var _self;
    // Define Method
    window.OutstreamPlayer = function(_c) {
        // Establish Reference to this
        _self = this;
        // Save Reference to Script Name
        _self.scriptName = "Outstream_Player";
        // Script Version
        var scriptVersion = "1.0.1";
        // Last Modified
        var lastModified = "2017-09-15";
        // Last Uploaded
        var lastUploaded = "2018-11-08";
        // Block Id
        _self.blockId = '6260';
        // Establish Config Object
        _self.config = _c;
        // Call Init Method once AdKit is loaded
        adkit.onReady(function() { _self.init(); });
    };

    OutstreamPlayer.prototype = {
        // Define global vars
        trackOnce: true,
        outstreamSuccessful:false,
        isIOS:false,
        isMobile:false,
        container: undefined,
        vastTag: undefined,
        videoTrackingModule: undefined,
        playing: false,
        visible: false,
        firstRun: true,
        firstPlay: true,
        firstPause: true,
        readyHidePlayButton: false,
        showReplay: false,
        width: undefined,
        height: undefined,
        vjs: undefined,
        // Set Defaults for options
        options: {
            "aspectRatio":"6:5",
            "isMuted": false,
            "autoPlay": false,
            "playOnVisible": false,
            "playInline": false,
            "visibilityThreshold": 50,
            "preroll": false,
            "postroll": false
        },
        // Function That Creates Element Var
        d: function(id) { return document.getElementById(id); },
        // Create Event Handlers for outstream class
        addEventListener: function(_e, _m, _b) { window.addEventListener(_e, _m, _b); },
        // Update Default options
        updateDefaultOptions: function() {
            // Loop through options
            for (var k in _self.options) {
                // Check to make sure both config and options object exists
                if (_self.options.hasOwnProperty(k) && _self.config.hasOwnProperty(k)) {
                    // Update Default
                    _self.options[k] = _self.config[k];
                }
            }
        },

        // Set up Script
        init: function() {
            // Fix for 4G connections where width is not registered
            _self.d(_self.config.container).style.width = '300px;';
            // Set Timeout - Fix for 4G Issues
            setTimeout(function() {
                // Check if player failed, try again
                if (!_self.outstreamSuccessful){ _self.buildOutStreamPlayer(); }
            }, 4000);

            // Check if iOS
            if (EB.API) {
                // Update iOS boolean
                _self.isIOS = EB.API.os.ios;
                // Update mobile boolean
                _self.isMobile = EB.API.os.mobile;
            }
            // Track Feature
            _self.trackAdFeatures('impression', 'AF_' + _self.scriptName + '_' + _self.blockId);
            // Update Defaults with options from config
            _self.updateDefaultOptions();

            // If play on visible option is set, add listener for visibility data
            if (_self.options.playOnVisible && EB.getVisibilityData().available !== false) {

                EB.addEventListener(EBG.EventName.VISIBILITY, _self.checkVisibility, false);
            } else {
                // Build Player
                _self.buildOutStreamPlayer();
            }
        },

        // Check Visibility
        checkVisibility: function() {
            // Grab Visibility Data from Delivery
            var percentVisible = EB.getVisibilityData().percentage;
            // Update Boolean
            if (percentVisible >= _self.options.visibilityThreshold && _self.options.playOnVisible) {
                // update boolean
                _self.visible = true;
                // check if player is paused
                if (!_self.playing && !_self.firstPlay && _self.vjs !== undefined) {
                    // Play player after timeout -fixes chrome promise bug
                    setTimeout(function() {
                        // Play player
                        _self.vjs.play();
                        // Update Player status
                        _self.playing = true;
                    }, 125);
                }
            } else if (percentVisible < _self.options.visibilityThreshold && _self.options.playOnVisible) {
                // Update boolean
                _self.visible = false;
                // check if player is playing
                if (_self.playing && _self.vjs !== undefined) {
                    // Pause player after timeout -fixes chrome promise bug
                    setTimeout(function() {
                        // Pause player
                        _self.vjs.pause();
                        // Update Player status
                        _self.playing = false;
                    }, 125);
                }
            }

            // Check if we've already built the player
            if (_self.firstRun) {
                // Update Boolean
                _self.firstRun = false;
                // Build Player
                _self.buildOutStreamPlayer();
            }
        },

        // Build Video Player and Place on page
        buildOutStreamPlayer: function() {
            // Establish var for event type
            var myEvent;
            // Determine Event type
            _self.isMobile ? myEvent = 'touchend' : myEvent = 'click';
            // Update div Id from Config Object
            _self.container = _self.d(_self.config.container);
            // Get Styles of Container Div
            var style = window.getComputedStyle(_self.container, null);
            // Update Width
            _self.width = parseInt(style.width, 10);
            // Update Height
            _self.height = parseInt(style.height, 10);
            // Create New Video Tag
            var posterTag = document.createElement("div");
            // Create New Video Tag
            var vidTag = document.createElement("video");
            // Create New Paragraph Tag
            var pTag = document.createElement("p");
            // Create New Source Tag
            var srcTag = document.createElement("source");
            // Update Id
            posterTag.id = "poster";
            // Update Id
            vidTag.id = "OutstreamPlayer";
            // Update Player Width
            vidTag.width = _self.width;
            // Update Player Height
            vidTag.height = _self.height;
            // Set autoPlay Attribute if requested
            //if (_self.options.autoPlay && !_self.options.playOnVisible) {
            if (_self.options.autoPlay) { vidTag.setAttribute("autoplay", ""); }
            // Set autoPlay Attribute if requested
            if (_self.options.autoPlay && EB.getVisibilityData().available === false) { vidTag.setAttribute("autoplay", ""); }
            // Set Attribute
            if (_self.options.isMuted) {
                // Update Mute on load to fix firefox bug - won't allow mute from DOM
                vidTag.onloadedmetadata = function() { this.muted = true; };
            }
            // Update Source Tage with type
            srcTag.setAttribute("type", "video/mp4");
            // Add Classes
            vidTag.setAttribute("class", "video-js vjs-default-skin vjs-big-play-centered");
            // Update Classes
            pTag.setAttribute("class", "vjs-no-js");
            // Update HTML
            pTag.innerHTML = "To view this video please enable JavaScript, and consider upgrading to a web browser that <a href=\"https://videojs.com/html5-video-support/\" target=\"_blank\">supports HTML5 video</a>";
            // Add Video Div to Container
            _self.container.appendChild(posterTag);
            // Add Video Div to Container
            _self.container.appendChild(vidTag);
            // Add Source Tag to Player
            vidTag.appendChild(srcTag);
            // Add Alt Message to Container
            vidTag.appendChild(pTag);
            // iOS 10 new playsinline attribute enable vidoe plays in line (muted)
            if (_self.options.playInline) { vidTag.setAttribute("playsinline", ""); }

            // Fix for Replay Button not working on Mac and Edge Browsers when autoplay is set to false
            // Listen for click on video container
            _self.container.addEventListener(myEvent, function(e) {
                // Establish Title of button clicked
                var el = e.target.title;
                // If no title Check parent title
                if (el == '') {
                    // determine element title
                    if (e.srcElement == undefined) { el = e.originalTarget.parentElement.firstElementChild.title; } else { el = e.srcElement.parentElement.title; }
                }

                // Verify the Pause button was clicked
                if (el === 'Pause' && _self.options.playOnVisible) {
                    // Update boolean so we no longer track visibility
                    _self.options.playOnVisible = false;
                } else if (el === 'Replay') {
                    // Update Play on visible boolean, stop tracking visibility
                    _self.options.playOnVisible = true;
                    // Set Timeout to play video if video did not play
                    setTimeout(function() { if(_self.vjs.paused()) { _self.vjs.play(); }}, 1000);
                } else if (el === 'Play Video') {
                    // Hide Big Button after first click
                    _self.vjs.bigPlayButton.dispose();
                }
            });

            // Track Video using Video Module
            videoTrackingModule = new EBG.VideoModule(vidTag);

            // Check if EB.API exists
            if (_self.isIOS) {
                // remove video poster for ios devices so it won't show up before video play or before show video play button
                vidTag.setAttribute("poster", "");
            }
            // Check if showing content before ad if not show dummy 1 frame video
            if (_self.config.content === '' || _self.config.content === undefined) {
                srcTag.setAttribute("src", "videos/1x1.mp4");
            } else {
                srcTag.setAttribute("src", _self.config.content);
            }

            // Player built, load Content from Provider
            _self.loadVideoJS();
        },

        // Handle VideoJS
        loadVideoJS: function() {
            // Update Boolean - report success
            _self.outstreamSuccessful = true;
            // Update vastTag from Config Object
            _self.vastTag = _self.config.vastTag;
            // Try to get vastTag from SV Data
            try {
                if (adkit.getSVData("vastTag")) {
                    _self.vastTag = adkit.getSVData("vastTag");
                }
            } catch (err) {}
            // Try to get vastTag from Line Id
            try {
                if (EB._adConfig.lineId.length > 5) {
                    _self.vastTag = EB._adConfig.lineId;
                }
            } catch (err) {}

            // initiate the videojs player
            _self.vjs = videojs("OutstreamPlayer", {
                "controls": true,
                "playsinline": _self.options.playInline,
                "autoplay": _self.options.autoplay,
                "muted": _self.options.isMuted,
                "preload": "auto",
                "fluid": true,
                "aspectRatio": _self.options.aspectRatio,
                "nativeControlsForTouch": false,
                "plugins": {
                    "vastClient": {
                        "adTagUrl": _self.vastTag,
                        "adCancelTimeout": 5000,
                        "adsEnabled": true,
                        "vpaidFlashLoaderPath": "https://szmk.io/VPAIDFlash.swf",
                        "playAdAlways": true,
                        "preferredTech": "html5",
                        "preroll": _self.options.preroll,
                        "postroll": _self.options.postroll,
                        "midrolls": _self.config.midrolls
                    }
                }
            }, function() {
                // If play on visible option is set and autoplay is true, the play video by checking visibility
                if (_self.options.playOnVisible && _self.options.autoPlay && EB.getVisibilityData().available !== false) {
                    // Update Boolean
                    _self.firstPlay = false;
                    // Check visibility
                    _self.checkVisibility();
                }
                // Get new reference to player for events not offered by videojs
                var vid = document.getElementById('OutstreamPlayer_html5_api');

                // Add Event Handlers for events not offered by videojs
                vid.addEventListener('seeking', _self.handleVideoSeeking, false);
                vid.addEventListener('seeked', _self.handleVideoSeekEnd, false);
                vid.addEventListener('durationchange', _self.handleVideoDurationChange, false);
                vid.addEventListener('volumechange', _self.handleVolumeChange, false);

                // Listen for Player Events from videojs
                this.on('play', function() { _self.handleVideoPlay(); });
                this.on('pause', function() { _self.handleVideoPause(); });
                this.on('ended', function() { _self.handleVideoEnd(); });
                this.on('timeupdate', function() { _self.handleTimeUpdate(); });
                this.on('loadedmetadata', function() { _self.handleMetaData(); });
                this.on('vast.contentStart', function() { _self.handleVastContentStart(); });
                this.on('vast.firstPlay', function() { _self.handleFirstPlay(); });
                this.on('vast.adStart', function() { _self.handleAdStart(); });
                this.on('vast.adSkip', function() { _self.handleAdSkip(); });
                this.on('vast.adEnd', function() { _self.handleAdEnd(); });
                this.on('vast.adError', function() { _self.handleAdErrors(); });
                this.on('vast.adsCancel', function() { _self.handleAdCancel(); });
                this.on('vast.adClickThru', function() { _self.handleAdClickThru(); });
            });
        },

        // Event Handlers
        handleVideoPlay: function() {
            _self.readyHidePlayButton = true;
            // Hide Big Play Button on first run but only if we know the video is actually playing
            setTimeout(function() {
                if (_self.readyHidePlayButton) {
                    // Hide Big Button after first click
                    _self.vjs.bigPlayButton.dispose();
                 }
             }, 1000);
            // Update first play Boolean
            _self.firstPlay = false;
            // Update Playing Status Boolean
            _self.playing = true;
            // Reset Play on Visible if replay button has been clicked
            if (_self.showReplay && _self.config.playOnVisible){
                // Update Play on visible boolean, start tracking visibility
                _self.options.playOnVisible = true;
            }
            // Update Replay Boolean
            _self.showReplay = false;
            // Check Ad Styling size
            _self.checkAdSize();
            // Fire Play Event
            _self.fireEvent('Video_Play');
        },

        handleVideoPause: function() {
            // Update Boolean
            _self.readyHidePlayButton = false;
            // Update Playing status boolean
            _self.playing = false;
            // Fire Pause Event
            _self.fireEvent('Video_Pause');
        },

        handleAdCancel: function() {
            // Update Boolean so we know replay button is showing
            _self.showReplay = true;
            // Update Play on visible boolean, stop tracking visibility
            _self.options.playOnVisible = false;
            // Fire Event
            _self.fireEvent('Ad_Cancel');
        },

        handleVideoSeeking: function() { _self.fireEvent('Video_Seeking'); },
        handleVideoSeekEnd: function() { _self.fireEvent('Video_Seek_End'); },
        handleVideoDurationChange: function() { _self.fireEvent('Video_Duration_Change'); },
        handleVolumeChange: function() { _self.fireEvent('Volume_Change'); },
        handleVideoEnd: function() { _self.fireEvent('Video_End'); },
        handleMetaData: function() { _self.fireEvent('Meta_Data'); },
        handleTimeUpdate: function() { _self.fireEvent('Time_Update'); },
        handleFirstPlay: function() { _self.fireEvent('First_Play'); },
        handleAdStart: function() { _self.fireEvent('Ad_Start'); },
        handleAdSkip: function() { _self.fireEvent('Ad_Skip'); },
        handleAdEnd: function() { _self.fireEvent('Ad_End'); },
        handleAdErrors: function() { _self.fireEvent('Ad_Error'); },
        handleVastContentStart: function() { _self.fireEvent('Vast_Content_Start'); },
        handleAdClickThru: function() { _self.fireEvent('Ad_Clickthrough'); },

        // Handle Dispatching events
        fireEvent: function(_name, _detail) {
            // Establish var for event
            var e = document.createEvent("Event");
            // Init Event
            e.initEvent(_name, true, false);
            // Pass Success Message
            e.detail = _detail;
            // Dispatch Event
            window.dispatchEvent(e);
        },

        // Make Play method available to dev
        play: function() { _self.vjs.play(); },
        // Make Pause method available to dev
        pause: function() { _self.vjs.pause(); },
        // Make Fullscreen method available to dev
        enterFullscreen: function() { _self.vjs.requestFullscreen(); },
        // Make Exit Fullscreen method available to dev
        exitFullscreen: function() { _self.vjs.exitFullscreen(); },
        // Make dispose method available to dev
        dispose: function() { _self.vjs.dispose(); },
        // Allow Dev to create new player
        create: function() { _self.buildOutStreamPlayer(); },

        // Apply fix for small resolutions where volume and advertiser label pushed fullscreen buttons off page
        checkAdSize: function() {
            // Check if video is less than 350px
            if (_self.width < 350){
                // Get Labels
                var lbl = document.getElementsByClassName('vjs-ads-label');
                // Wait till label
                setTimeout(function(){
                    // Loop through labels
                    for (var i = 0; i < lbl.length; i++) {
                        // Apply style for small displays
                        lbl[i].setAttribute('style', 'font-size:9px; padding-left:5px;');
                    }
                }, 100);
            }
        },

        // DO NOT REMOVE THE FOLLOWING
        // Track Ad Features - accepts interaction name and impression type
        trackAdFeatures: function(_interaction, _noun) {
            // Establish var to track local mode
            var isServed;
            // Verify ad is not local
            try { isServed = EB._isServingMode(); } catch (err) {}

            if (isServed && _self.trackOnce) {
                try {
                    // Update Boolean
                    _self.trackOnce = false;
                    // Grab Ad ID
                    var adId = EB._adConfig.adId;
                    // Grab Session ID
                    var sId = EB._adConfig.sID;
                    // Build Tracking Pixel
                    var trackingPixel = "https://bs.serving-sys.com/BurstingPipe/adServer.bs?cn=tf&c=19&mc=imp&pli=16479316&PluID=0&ord=%time%&rtu=-1&pcp=$$sID=" + sId + "|adID=" + adId + "|interactionName=" + _interaction + "|noun=" + _noun + "$$";
                    // Fire Tracking Pixel by creating a new image.
                    new Image().src = trackingPixel;

                } catch (err) {}
            }
        }
    };
}());
