/*******************
VARIABLES
*******************/
var creativeId = "HTMLResponsiveRichMediaBanner";
var creativeVersion = "1.1.0";
var lastModified = "2017-02-07";
var lastUploaded = "2017-02-07";
var templateVersion = "2.0.20";
var bannerDiv;
var adId, rnd, uid;
var dimensions;
var scrollPos = {x:undefined, y:undefined};
var video;
var primaryContent;
var outstream;
var videoContainer;
var videoContinue = true;
var tempPlayOnVisible;
var checkedPlayOnVisible = false;

/*******************
INITIALIZATION
*******************/
function checkIfAdKitReady(event) {
	try{
		if (window.localPreview) {
			window.initializeLocalPreview(); //in localPreview.js
			initializeCreative();
			return;
		}
	}catch(e){}
    adkit.onReady(initializeCreative);
}

function initializeCreative(event) {
	// Instantiate Outstream Player
	outstream = new OutstreamPlayer(config);
	// Start Outstream Player if in local Preview
	if (window.localPreview) { outstream.init(); }
	// Save Option Preference
	tempPlayOnVisible = outstream.options.playOnVisible;
	// Listen for outstream events
	outstream.addEventListener('Ad_Cancel', updateVisibility, false);
	//Workaround (from QB6573) for Async EB Load where Modernizr isn't properly initialized
	typeof Modernizr == "object" && (Modernizr.touch = Modernizr.touch || "ontouchstart" in window);

    initializeGlobalVariables();
    //initializeVideo();
    addEventListeners();
    displayDimensions();
    setCreativeVersion();
}

// Called when replay button is shown - we want to stop tracking visibility
function updateVisibility() {
	// Stop Tracking Video
	videoContinue = false;
}

function initializeGlobalVariables() {
	adId = EB._adConfig.adId;
	rnd = EB._adConfig.rnd;
	uid = EB._adConfig.uid;
    dimensions = document.getElementById("dimensions");
    video = document.getElementById("OutstreamPlayer_html5_api");
    primaryContent = document.getElementById("primary-content");
	bannerDiv = document.getElementById("banner");
	videoContainer = document.getElementById(config.container);
}

function initializeVideo() {
    var videoTracker = new EBG.VideoModule(video);

    video.classList.remove("hide");


    var primaryContent = document.getElementById("primary-content");
    primaryContent.classList.remove("hide");
}

function addEventListeners() {
    document.getElementById("user-action-button").addEventListener("click", handleUserActionButtonClick);
    document.getElementById("clickthrough-button").addEventListener("click", handleClickthroughButtonClick);

    window.addEventListener("resize", checkIfVideoIsVisible);
    window.addEventListener("resize", displayDimensions);

	// Check if iOS
	var isMobile = false;

	if (EB.API) { isMobile = EB.API.os.mobile; }
	// Establish var for event type
	var myEvent;
	// Determine Event type
	isMobile ? myEvent = 'touchend' : myEvent = 'click';

	// Listen for click on video container
	videoContainer.addEventListener(myEvent, function(e) {
        // Establish Title of button clicked
        var el = e.target.title;
        // If no title Check parent title
        if (el == '') {
            // determine element title
            if (e.srcElement == undefined) { el = e.originalTarget.parentElement.firstElementChild.title; } else { el = e.srcElement.parentElement.title; }
        }
		// Verify the Pause button was clicked
		if (el === 'Pause') {
			// Update boolean so we no longer track visibility
			videoContinue = false;
		} else if (el === 'Play' || el === 'Replay') {
			videoContinue = true;
		}
	});
}

/*******************
EVENT HANDLERS
*******************/
function handleUserActionButtonClick(event) {
    EB.userActionCounter("UserAction");
}

function handleClickthroughButtonClick(event) {
	videoContinue = false;
    pauseVideo();
    EB.clickthrough();
}

function checkIfVideoIsVisible(event) {
    if (primaryContent) {
        var isContentVisible = window.getComputedStyle(primaryContent).display !== "none";

        if (!isContentVisible) {
            pauseVideo();
        } else if (videoContinue) {
			playVideo();
		}
    }
}

function pauseVideo() {
	// get video element
	var video = document.getElementById("OutstreamPlayer_html5_api");
	// Save option preference - only do it once
	if (!checkedPlayOnVisible) {
		// Save Preference
		tempPlayOnVisible = outstream.options.playOnVisible;
		// Update Boolean
		checkedPlayOnVisible = true;
	}
	// stop tracking visibility
	outstream.options.playOnVisible = false;
	// pause video
	try { video.pause(); } catch (e){}
}

function playVideo() {
	// get video element
	var video = document.getElementById("OutstreamPlayer_html5_api");
	// reset option preference
	outstream.options.playOnVisible = tempPlayOnVisible;
	// Update Boolean
	checkedPlayOnVisible = false;
	// pause video
	try { video.play(); } catch (e){}
}

function displayDimensions(event) {
    if (dimensions) {
        var str = window.innerWidth + "x" + window.innerHeight;
        dimensions.innerHTML = str;
    }
}

function onPageScroll(event) {
	// use scrollPos anywhere to know the current x/y coordinates.
	scrollPos.x = event.scrollXPercent;
	scrollPos.y = event.scrollYPercent;
}

/*******************
UTILITIES
*******************/
function setCreativeVersion() {
	EB._sendMessage("setCreativeVersion", {
		creativeId:				creativeId,
		creativeVersion:		creativeVersion,
		creativeLastModified:	lastModified,
		uid:					uid
	});
}

/*********************************
HTML5 Event System - Do Not Modify
*********************************/
var listenerQueue;
var creativeIFrameId;

function addCustomScriptEventListener(eventName, callback, interAd) {
	listenerQueue = listenerQueue || {};
	var data = {
		uid				: uid,
		listenerId		: Math.ceil(Math.random() * 1000000000),
		eventName		: eventName,
		interAd			: !!(interAd),
		creativeIFrameId: creativeIFrameId
	};
	EB._sendMessage("addCustomScriptEventListener", data);
	data.callback = callback;
	listenerQueue[data.listenerId] = data;
	return data.listenerId;
}

function dispatchCustomScriptEvent(eventName, params) {
	params 					= params || {};
	params.uid 				= uid;
	params.eventName 		= eventName;
	params.creativeIFrameId	= creativeIFrameId;
	EB._sendMessage("dispatchCustomScriptEvent", params);
}

function removeCustomScriptEventListener(listenerId) {
	var params = {
		uid 			: uid,
		listenerId 		: listenerId,
		creativeIFrameId: creativeIFrameId
	};

	EB._sendMessage("removeCustomScriptEventListener", params);
	if (listenerQueue[listenerId])
	delete listenerQueue[listenerId];
}

function eventManager(event) {
	var msg = JSON.parse(event.data);
	if(msg.type && msg.data && (!uid || (msg.data.uid && msg.data.uid == uid))) {
		switch (msg.type) {
			case "sendCreativeId":
				creativeIFrameId = msg.data.creativeIFrameId;
				addCustomScriptEventListener('pageScroll', onPageScroll);
				EB._sendMessage("dispatchScrollPos",{uid:					uid});
				if(creativeContainerReady)
					creativeContainerReady();
			break;
			case "eventCallback": // Handle Callback
				var list 	= msg.data.listenerIds;
				var length  = list.length;
				for (var i = 0; i < length; i++){
					try {
						var t = listenerQueue[list[i]];
						if (!t) continue;
						t.callback(msg.data);
					} catch (e){}
				}
			break;
		}
	}
}

window.addEventListener("message",function(){try{eventManager.apply(this, arguments);}catch(e){}},false);
/*************************************
End HTML5 Event System - Do Not Modify
*************************************/

window.addEventListener("load", checkIfAdKitReady);
